@@ -1,4 +1,4 @@
-/* Copyright (c) 2016, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -23,6 +23,10 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 #ifndef _mysql_sql_command_h
 #define _mysql_sql_command_h
 
+/**
+  @file include/my_sqlcommand.h
+*/
+
 /*
   @enum  enum_sql_command
   @brief SQL Commands
@@ -167,7 +171,6 @@ enum enum_sql_command {
   SQLCOM_SHOW_CREATE_EVENT,
   SQLCOM_SHOW_EVENTS,
   SQLCOM_SHOW_CREATE_TRIGGER,
-  SQLCOM_ALTER_DB_UPGRADE,
   SQLCOM_SHOW_PROFILE,
   SQLCOM_SHOW_PROFILES,
   SQLCOM_SIGNAL,
@@ -178,24 +181,39 @@ enum enum_sql_command {
   SQLCOM_EXPLAIN_OTHER,
   SQLCOM_SHOW_CREATE_USER,
   SQLCOM_SHUTDOWN,
+  SQLCOM_SET_PASSWORD,
   SQLCOM_ALTER_INSTANCE,
+  SQLCOM_INSTALL_COMPONENT,
+  SQLCOM_UNINSTALL_COMPONENT,
+  SQLCOM_CREATE_ROLE,
+  SQLCOM_DROP_ROLE,
+  SQLCOM_SET_ROLE,
+  SQLCOM_GRANT_ROLE,
+  SQLCOM_REVOKE_ROLE,
+  SQLCOM_ALTER_USER_DEFAULT_ROLE,
+  SQLCOM_IMPORT,
+  SQLCOM_CREATE_RESOURCE_GROUP,
+  SQLCOM_ALTER_RESOURCE_GROUP,
+  SQLCOM_DROP_RESOURCE_GROUP,
+  SQLCOM_SET_RESOURCE_GROUP,
+  SQLCOM_CLONE,
+  SQLCOM_LOCK_INSTANCE,
+  SQLCOM_UNLOCK_INSTANCE,
+  SQLCOM_RESTART_SERVER,
+  SQLCOM_CREATE_SRS,
+  SQLCOM_DROP_SRS,
   SQLCOM_SHOW_USER_STATS,
   SQLCOM_SHOW_TABLE_STATS,
   SQLCOM_SHOW_INDEX_STATS,
   SQLCOM_SHOW_CLIENT_STATS,
   SQLCOM_SHOW_THREAD_STATS,
   SQLCOM_LOCK_TABLES_FOR_BACKUP,
-  SQLCOM_LOCK_BINLOG_FOR_BACKUP,
-  SQLCOM_UNLOCK_BINLOG,
   SQLCOM_CREATE_COMPRESSION_DICTIONARY,
   SQLCOM_DROP_COMPRESSION_DICTIONARY,
-
 #ifdef WITH_WSREP
-  /*
-    Also for PXC, if the command is replicated to other nodes as a
-    DDL (no-writeset replication), add a testcase to
-    galera_wsrep_ddl_access_checking.test
-  */
+  /* Placeholder. If you see more command following this
+  lines then check if they need handling in PXC.
+  PXC has special handling (TOI based) for DDL command. */
 #endif /* WITH_WSREP */
   /* This should be the last !!! */
   SQLCOM_END
