@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2017 Codership Oy <info@codership.com>
+ * Copyright (C) 2008-2020 Codership Oy <info@codership.com>
  *
  * Queue (FIFO) class implementation
  *
@@ -339,14 +339,19 @@ void* gu_fifo_get_head (gu_fifo_t* q, int* err)
     }
 }
 
-/*! Advances FIFO head and unlocks FIFO. */
-void gu_fifo_pop_head (gu_fifo_t* q)
+/*! Unprotected helper for gu_fifo_pop_head() and gu_fifo_clear() */
+static inline
+void fifo_advance_head (gu_fifo_t* q)
 {
+#ifdef PXC
     if ((FIFO_COL(q, q->head) == q->col_mask) &&
-        (FIFO_ROW(q, q->head) != FIFO_ROW(q, q->tail)))
-     {
+        (FIFO_ROW(q, q->head) != FIFO_ROW(q, q->tail))) {
         /* free the row if we are removing the last unit from the row and
          * if the tail is not in the same row as the head */
+#else
+    if (FIFO_COL(q, q->head) == q->col_mask) {
+        /* removing last unit from the row */
+#endif /* PXC */
         ulong row = FIFO_ROW (q, q->head);
         assert (q->rows[row] != NULL);
         gu_free (q->rows[row]);
@@ -359,6 +364,12 @@ void gu_fifo_pop_head (gu_fifo_t* q)
     if (gu_unlikely(q->used < q->used_min)) {
         q->used_min = q->used;
     }
+}
+
+/*! Advances FIFO head and unlocks FIFO. */
+void gu_fifo_pop_head (gu_fifo_t* q)
+{
+    fifo_advance_head(q);
 
     if (fifo_unlock_get(q)) {
         gu_fatal ("Failed to unlock queue to get item.");
@@ -429,6 +440,7 @@ long gu_fifo_length (gu_fifo_t* q)
     return q->used;
 }
 
+#ifdef PXC
 /*! Returns the maximum number of items allowed in the queue */
 long gu_fifo_max_length (gu_fifo_t* q)
 {
@@ -437,6 +449,7 @@ long gu_fifo_max_length (gu_fifo_t* q)
     */
     return q->length-1;
 }
+#endif /* PXC */
 
 /*! returns how many items were in the queue per push_tail() */
 void gu_fifo_stats_get (gu_fifo_t* q, int* q_len, int* q_len_max,
@@ -481,6 +494,15 @@ void gu_fifo_stats_flush(gu_fifo_t* q)
     fifo_unlock (q);
 }
 
+void gu_fifo_clear(gu_fifo_t* q)
+{
+    fifo_lock (q);
+
+    while (q->used > 0) fifo_advance_head(q);
+
+    fifo_unlock (q);
+}
+
 /* destructor - would block until all members are dequeued */
 void gu_fifo_destroy   (gu_fifo_t *queue)
 {
@@ -514,6 +536,9 @@ void gu_fifo_destroy   (gu_fifo_t *queue)
     {
         ulong row = FIFO_ROW(queue, queue->tail);
         if (queue->rows[row]) {
+#ifndef PXC
+            assert (FIFO_COL(queue, queue->tail) != 0);
+#endif /* !PXC */
             gu_free (queue->rows[row]);
             queue->alloc -= queue->row_size;
         }
