@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009-2018 Codership Oy <info@codership.com>
+ * Copyright (C) 2009-2021 Codership Oy <info@codership.com>
  */
 
 #include "GCache.hpp"
@@ -21,51 +21,69 @@ namespace gcache
 
         mallocs  = 0;
         reallocs = 0;
+#ifdef PXC
         frees    = 0;
+#endif /* PXC */
 
-        seqno_locked   = SEQNO_NONE;
+        gid            = gu::UUID();
         seqno_max      = SEQNO_NONE;
         seqno_released = SEQNO_NONE;
-        gid            = gu::UUID();
+        seqno_locked   = SEQNO_MAX;
+        seqno_locked_count = 0;
 
-        seqno2ptr.clear();
+        seqno2ptr.clear(SEQNO_NONE);
 
 #ifndef NDEBUG
         buf_tracker.clear();
 #endif
     }
 
-    GCache::GCache (gu::Config& cfg, const std::string& data_dir)
+    GCache::GCache (ProgressCallback* pcb,
+                    gu::Config& cfg, const std::string& data_dir,
+                    gu::MasterKeyProvider* mk_provider)
         :
         config    (cfg),
         params    (config, data_dir),
+#ifdef PXC
 #ifdef HAVE_PSI_INTERFACE
         mtx       (WSREP_PFS_INSTR_TAG_GCACHE_MUTEX),
-        cond      (WSREP_PFS_INSTR_TAG_GCACHE_CONDVAR),
+#else
+         mtx       (),
+#endif /* HAVE_PSI_INTERFACE */
 #else
         mtx       (),
-        cond      (),
-#endif /* HAVE_PSI_INTERFACE */
-        seqno2ptr (),
+#endif /* PXC */
+        seqno2ptr (SEQNO_NONE),
         gid       (),
         mem       (params.mem_size(), seqno2ptr, params.debug()),
-        rb        (params.rb_name(), params.rb_size(), seqno2ptr, gid,
-                   params.debug(), params.recover()),
+        rb        (pcb, params.rb_name(), params.rb_size(), seqno2ptr, gid,
+                   params.debug(), params.recover(), params.encrypt(),
+                   params.encryption_cache_page_size(),
+                   std::min(params.encryption_cache_size(), params.rb_size()),
+                   mk_provider),
         ps        (params.dir_name(),
                    params.keep_pages_size(),
                    params.page_size(),
                    params.debug(),
+#ifdef PXC
                    /* keep last page if PS is the only storage */
                    params.keep_pages_count() ?
                    params.keep_pages_count() :
+                    !((params.mem_size() + params.rb_size()) > 0),
+                   params.encrypt(),
+                   params.encryption_cache_page_size(),
+                   std::min(params.encryption_cache_size(), params.page_size())),
+#else
                    !((params.mem_size() + params.rb_size()) > 0)),
+#endif /* PXC */
         mallocs   (0),
         reallocs  (0),
         frees     (0),
-        seqno_locked(SEQNO_NONE),
-        seqno_max   (seqno2ptr.empty() ?
-                     SEQNO_NONE : seqno2ptr.rbegin()->first),
-        seqno_released(seqno_max)
+        seqno_max     (seqno2ptr.empty() ?
+                       SEQNO_NONE : seqno2ptr.index_back()),
+        seqno_released(seqno_max),
+        seqno_locked  (SEQNO_MAX),
+        seqno_locked_count(0)
 #ifndef NDEBUG
         ,buf_tracker()
 #endif
@@ -79,6 +97,7 @@ namespace gcache
                   << "\n" << "GCache frees   : " << frees;
     }
 
+#ifdef PXC
     size_t GCache::allocated_pool_size ()
     {
         gu::Lock lock(mtx);
@@ -86,6 +105,7 @@ namespace gcache
                rb.allocated_pool_size() +
                ps.allocated_pool_size();
     }
+#endif /* PXC */
 
     /*! prints object properties */
     void print (std::ostream& os) {}
@@ -95,8 +115,9 @@ namespace gcache
 
 gcache_t* gcache_create (gu_config_t* conf, const char* data_dir)
 {
+    /* this funciton is used only in tests */
     gcache::GCache* gc = new gcache::GCache (
-        *reinterpret_cast<gu::Config*>(conf), data_dir);
+        NULL, *reinterpret_cast<gu::Config*>(conf), data_dir);
     return reinterpret_cast<gcache_t*>(gc);
 }
 
@@ -129,24 +150,3 @@ int64_t gcache_seqno_min (gcache_t* gc)
     gcache::GCache* gcache = reinterpret_cast<gcache::GCache*>(gc);
     return gcache->seqno_min ();
 }
-
-#if DEPRECATED
-void  gcache_seqno_init   (gcache_t* gc, int64_t seqno)
-{
-    gcache::GCache* gcache = reinterpret_cast<gcache::GCache*>(gc);
-    gcache->seqno_init (seqno);
-}
-
-void  gcache_seqno_assign(gcache_t* gc, const void* ptr, int64_t seqno)
-{
-    gcache::GCache* gcache = reinterpret_cast<gcache::GCache*>(gc);
-    gcache->seqno_assign (ptr, seqno, -1, false);
-}
-
-void  gcache_seqno_release(gcache_t* gc, const void* ptr)
-{
-    gcache::GCache* gcache = reinterpret_cast<gcache::GCache*>(gc);
-    gcache->seqno_release ();
-}
-#endif
-
