@@ -12,6 +12,7 @@
 #include "gu_macros.h"
 #include <algorithm>
 #include <set>
+#include <iostream> // std::cerr
 
 #include <boost/bind.hpp>
 
@@ -65,6 +66,30 @@ public:
     }
 };
 
+class UUIDFixedPartCmp
+{
+public:
+    UUIDFixedPartCmp(const gcomm::UUID& uuid) : uuid_(uuid) { }
+    bool operator()(const gcomm::NodeList::value_type& vt) const
+    {
+        return uuid_.fixed_part_matches(vt.first);
+    }
+private:
+    const gcomm::UUID& uuid_;
+};
+
+static bool UUID_fixed_part_cmp_equal(const gcomm::NodeList::value_type& lhs,
+                                      const gcomm::NodeList::value_type& rhs)
+{
+    return lhs.first.fixed_part_matches(rhs.first);
+}
+
+static bool UUID_fixed_part_cmp_intersection(const gcomm::UUID& lhs,
+                                             const gcomm::UUID& rhs)
+{
+    return lhs.fixed_part_matches(rhs) ? false : lhs < rhs;
+}
+
 // Return max to seq found from states, -1 if states is empty
 static int64_t get_max_to_seq(const gcomm::pc::Proto::SMMap& states)
 {
@@ -243,8 +268,11 @@ void gcomm::pc::Proto::deliver_view(bool bootstrap)
             i != instances_.end(); ++i) {
             const UUID& uuid(NodeMap::key(i));
             // just consider property of nodes in restored view.
-            if (rst_view_ -> members().find(uuid) !=
-                rst_view_ -> members().end()) {
+            if (std::find_if(rst_view_->members().begin(),
+                             rst_view_->members().end(),
+                             UUIDFixedPartCmp(uuid))
+                != rst_view_->members().end())
+            {
                 const Node& node(NodeMap::value(i));
                 const ViewId& last_prim(node.last_prim());
                 if (last_prim.type() != V_NON_PRIM ||
@@ -270,16 +298,18 @@ void gcomm::pc::Proto::deliver_view(bool bootstrap)
             log_debug << "deliver_view = ";
             log_debug << v;
 
-            if (rst_view_ -> id().seq() == max_view_seqno &&
-                rst_view_ -> members() == v.members()) {
+            if (rst_view_->id().seq() == max_view_seqno &&
+                v.members().size() == rst_view_->members().size() &&
+                std::equal(v.members().begin(), v.members().end(),
+                           rst_view_->members().begin(), UUID_fixed_part_cmp_equal))
+            {
                 log_info << "promote to primary component";
-                // since all of them are non-primary component
-                // we need to bootstrap.
+                // All of the nodes are in non-primary so we need to bootstrap.
                 send_install(true);
-                // clear rst_view after pc is formed, otherwise
-                // there would be network partition when sending
-                // install message. and if rst_view is cleared here,
-                // then pc recovery will never happen again.
+                // Rst_view will be cleared after primary component is formed.
+                // If the rst_view would be cleared here and there would be
+                // network partitioning before install message was delivered,
+                // bootstrapping the primary component would never happen again.
             }
         }
     }
@@ -321,18 +351,13 @@ void gcomm::pc::Proto::shift_to(const State s)
     // State graph
     static const bool allowed[S_MAX][S_MAX] = {
 
-        // Closed
-        { false, false,  false, false, false, true },
-        // States exch
-        { true,  false, true,  false, true,  true  },
-        // Install
-        { true,  false, false, true,  true,  true  },
-        // Prim
-        { true,  false, false, false, true,  true  },
-        // Trans
-        { true,  true,  false, false, false, true  },
-        // Non-prim
-        { true,  false,  false, true, true,  true  }
+        // Cl     S-E    IN     P      Trans  N-P
+        {  false, false, false, false, false, true  }, // Closed
+        {  true,  false, true,  false, true,  true  }, // States exch
+        {  true,  false, false, true,  true,  true  }, // Install
+        {  true,  false, false, false, true,  true  }, // Prim
+        {  true,  true,  false, false, false, true  }, // Trans
+        {  true,  false, false,  true, true,  true  }  // Non-prim
     };
 
 
@@ -476,20 +501,40 @@ static bool have_weights(const gcomm::NodeList& node_list,
     return true;
 }
 
+static bool node_list_intersection_comp(const gcomm::NodeList::value_type& vt1,
+                                        const gcomm::NodeList::value_type& vt2)
+{
+    return (vt1.first < vt2.first);
+}
+
+static gcomm::NodeList node_list_intersection(const gcomm::NodeList& nl1,
+                                              const gcomm::NodeList& nl2)
+{
+    gcomm::NodeList ret;
+    std::set_intersection(nl1.begin(), nl1.end(), nl2.begin(), nl2.end(),
+                          std::inserter(ret, ret.begin()),
+                          node_list_intersection_comp);
+    return ret;
+}
 
 bool gcomm::pc::Proto::have_quorum(const View& view, const View& pc_view) const
 {
+    // Compare only against members and left which were part of the pc_view.
+    gcomm::NodeList memb_intersection(
+        node_list_intersection(view.members(), pc_view.members()));
+    gcomm::NodeList left_intersection(
+        node_list_intersection(view.left(), pc_view.members()));
     if (have_weights(view.members(), instances_) &&
         have_weights(view.left(), instances_)    &&
         have_weights(pc_view.members(), instances_))
     {
-        return (weighted_sum(view.members(), instances_) * 2
-                + weighted_sum(view.left(), instances_) >
+        return (weighted_sum(memb_intersection, instances_) * 2
+                + weighted_sum(left_intersection, instances_) >
                 weighted_sum(pc_view.members(), instances_));
     }
     else
     {
-        return (view.members().size()*2 + view.left().size() >
+        return (memb_intersection.size()*2 + left_intersection.size() >
                 pc_view.members().size());
     }
 }
@@ -497,17 +542,22 @@ bool gcomm::pc::Proto::have_quorum(const View& view, const View& pc_view) const
 
 bool gcomm::pc::Proto::have_split_brain(const View& view) const
 {
+    // Compare only against members and left which were part of the pc_view.
+    gcomm::NodeList memb_intersection(
+        node_list_intersection(view.members(), pc_view_.members()));
+    gcomm::NodeList left_intersection(
+        node_list_intersection(view.left(), pc_view_.members()));
     if (have_weights(view.members(), instances_)  &&
         have_weights(view.left(), instances_)     &&
         have_weights(pc_view_.members(), instances_))
     {
-        return (weighted_sum(view.members(), instances_) * 2
-                + weighted_sum(view.left(), instances_) ==
+        return (weighted_sum(memb_intersection, instances_) * 2
+                + weighted_sum(left_intersection, instances_) ==
                 weighted_sum(pc_view_.members(), instances_));
     }
     else
     {
-        return (view.members().size()*2 + view.left().size() ==
+        return (memb_intersection.size()*2 + left_intersection.size() ==
                 pc_view_.members().size());
     }
 }
@@ -523,7 +573,7 @@ void gcomm::pc::Proto::handle_trans(const View& view)
     log_debug << self_id() << " \n\n current view " << current_view_
               << "\n\n next view " << view
               << "\n\n pc view " << pc_view_;
-
+    log_debug << *this;
     if (have_quorum(view, pc_view_) == false)
     {
         if (closing_ == false && ignore_sb_ == true && have_split_brain(view))
@@ -953,7 +1003,8 @@ bool gcomm::pc::Proto::is_prim() const
         std::set<UUID> intersection;
         set_intersection(greatest_view.begin(), greatest_view.end(),
                          present.begin(), present.end(),
-                         inserter(intersection, intersection.begin()));
+                         inserter(intersection, intersection.begin()),
+                         UUID_fixed_part_cmp_intersection);
         log_debug << self_id()
                   << " intersection size " << intersection.size()
                   << " greatest view size " << greatest_view.size();
