@@ -91,6 +91,7 @@ private:
 
 public:
 
+#ifdef PXC
     RecvBuf()
         :
 #ifdef HAVE_PSI_INTERFACE
@@ -101,6 +102,9 @@ public:
         cond_(),
 #endif /* HAVE_PSI_INTERFACE */
         queue_(), waiting_(false) { }
+#else
+    RecvBuf() : mutex_(), cond_(), queue_(), waiting_(false) { }
+#endif /* PXC */
 
     void push_back(const RecvBufData& p)
     {
@@ -141,6 +145,7 @@ public:
 
 private:
 
+#ifdef PXC
 #ifdef HAVE_PSI_INTERFACE
     gu::MutexWithPFS mutex_;
     gu::CondWithPFS cond_;
@@ -148,6 +153,10 @@ private:
     gu::Mutex mutex_;
     gu::Cond cond_;
 #endif /* HAVE_PSI_INTERFACE */
+#else
+    gu::Mutex mutex_;
+    gu::Cond cond_;
+#endif /* PXC */
     RecvBufQueue queue_;
     bool waiting_;
 };
@@ -166,22 +175,53 @@ public:
         uri_(u),
         net_(Protonet::create(conf_)),
         tp_(0),
+#ifdef PXC
 #ifdef HAVE_PSI_INTERFACE
         mutex_(WSREP_PFS_INSTR_TAG_GCOMMCONN_MUTEX),
 #else
-        mutex_(),
+         mutex_(),
 #endif /* HAVE_PSI_INTERFACE */
+#else
+        mutex_(),
+#endif /* PXC */
         refcnt_(0),
         terminated_(false),
         error_(0),
         recv_buf_(),
         current_view_()
     {
-        log_debug << "backend: " << net_->type();
+        log_info << "backend: " << net_->type();
     }
 
     ~GCommConn()
     {
+        // We cannot call gcs_close() to perform cleanup if there is an
+        // exception. So, we need to explicity free the objects in the
+        // destructor.
+        if (tp_ != nullptr)
+        {
+            delete tp_;
+        }
+
+        bool unlock_needed = true;
+        mutex_.lock();
+        if (!terminated_)
+        {
+            {
+                gcomm::Critical<Protonet> crit(*net_);
+                log_info << "gcomm: terminating thread";
+                terminated_ = true;
+                notify();
+            }
+            mutex_.unlock();
+            unlock_needed = false;
+            log_info << "gcomm: joining thread";
+            gu_thread_join(thd_, 0);
+        }
+        if (unlock_needed)
+        {
+            mutex_.unlock();
+        }
         delete net_;
     }
 
@@ -204,20 +244,22 @@ public:
             terminate();
         }
         log_info << "gcomm: joining thread";
-        pthread_join(thd_, 0);
+        gu_thread_join(thd_, 0);
         {
             gcomm::Critical<Protonet> crit(*net_);
+#ifdef PXC
             if (tp_ == 0)
             {
                 log_info << "gcomm: backend closed already";
             }
             else
+#endif /* PXC */
             {
-                log_info << "gcomm: closing backend";
-                tp_->close(error_ != 0 || force == true);
-                gcomm::disconnect(tp_, this);
-                delete tp_;
-                tp_ = 0;
+            log_info << "gcomm: closing backend";
+            tp_->close(error_ != 0 || force == true);
+            gcomm::disconnect(tp_, this);
+            delete tp_;
+            tp_ = 0;
             }
         }
         log_info << "gcomm: closed";
@@ -309,17 +351,21 @@ private:
 
     gu::Config&       conf_;
     gcomm::UUID       uuid_;
-    pthread_t         thd_;
+    gu_thread_t       thd_;
     ThreadSchedparam  schedparam_;
     Barrier           barrier_;
     URI               uri_;
     Protonet*         net_;
     Transport*        tp_;
+#ifdef PXC
 #ifdef HAVE_PSI_INTERFACE
     gu::MutexWithPFS  mutex_;
 #else
     gu::Mutex         mutex_;
 #endif /* HAVE_PSI_INTERFACE */
+#else
+    gu::Mutex         mutex_;
+#endif /* PXC */
     size_t            refcnt_;
     bool              terminated_;
     int               error_;
@@ -330,21 +376,25 @@ private:
 extern "C"
 void* run_fn(void* arg)
 {
+#ifdef PXC
 #ifdef HAVE_PSI_INTERFACE
-    pfs_instr_callback(WSREP_PFS_INSTR_TYPE_THREAD,
-                       WSREP_PFS_INSTR_OPS_INIT,
-                       WSREP_PFS_INSTR_TAG_GCOMMCONN_THREAD,
-                       NULL, NULL, NULL);
+        pfs_instr_callback(WSREP_PFS_INSTR_TYPE_THREAD,
+                           WSREP_PFS_INSTR_OPS_INIT,
+                           WSREP_PFS_INSTR_TAG_GCOMMCONN_THREAD,
+                           NULL, NULL, NULL);
 #endif /* HAVE_PSI_INTERFACE */
+#endif /* PXC */
 
     static_cast<GCommConn*>(arg)->run();
 
+#ifdef PXC
 #ifdef HAVE_PSI_INTERFACE
-    pfs_instr_callback(WSREP_PFS_INSTR_TYPE_THREAD,
-                       WSREP_PFS_INSTR_OPS_DESTROY,
-                       WSREP_PFS_INSTR_TAG_GCOMMCONN_THREAD,
-                       NULL, NULL, NULL);
+        pfs_instr_callback(WSREP_PFS_INSTR_TYPE_THREAD,
+                           WSREP_PFS_INSTR_OPS_DESTROY,
+                           WSREP_PFS_INSTR_TAG_GCOMMCONN_THREAD,
+                           NULL, NULL, NULL);
 #endif /* HAVE_PSI_INTERFACE */
+#endif /* PXC */
     gu_thread_exit(0);
 }
 
@@ -565,8 +615,8 @@ static GCS_BACKEND_SEND_FN(gcomm_send)
     {
         try
         {
-            orig_sp = gu::thread_get_schedparam(pthread_self());
-            gu::thread_set_schedparam(pthread_self(), conn.schedparam());
+            orig_sp = gu::thread_get_schedparam(gu_thread_self());
+            gu::thread_set_schedparam(gu_thread_self(), conn.schedparam());
         }
         catch (gu::Exception& e)
         {
@@ -594,7 +644,7 @@ static GCS_BACKEND_SEND_FN(gcomm_send)
     {
         try
         {
-            gu::thread_set_schedparam(pthread_self(), orig_sp);
+            gu::thread_set_schedparam(gu_thread_self(), orig_sp);
         }
         catch (gu::Exception& e)
         {
