@@ -14,28 +14,24 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
 
-
 #ifndef AUDIT_LOG_BUFFER_INCLUDED
 #define AUDIT_LOG_BUFFER_INCLUDED
 
-#include <string.h> // for size_t
+#include <stddef.h>
+#include "logger.h"
 
-#ifdef __cplusplus
-extern "C" {
-#endif
+struct audit_log_buffer_t;
 
-typedef struct audit_log_buffer audit_log_buffer_t;
+typedef int (*audit_log_write_func)(void *data, const char *buf, size_t len,
+                                    log_record_state_t state);
 
-typedef int (*audit_log_write_func)(void *data, const char *buf, size_t len);
-
-audit_log_buffer_t *audit_log_buffer_init(size_t size, int drop_if_full,
-                                 audit_log_write_func write_func, void *data);
-void audit_log_buffer_shutdown(audit_log_buffer_t *log);
-int audit_log_buffer_write(audit_log_buffer_t *log,
-                           const char *buf, size_t len);
-
-#ifdef __cplusplus
-}
-#endif
+audit_log_buffer_t *audit_log_buffer_init(size_t size, bool drop_if_full,
+                                          audit_log_write_func write_func,
+                                          void *data) noexcept;
+void audit_log_buffer_shutdown(audit_log_buffer_t *log) noexcept;
+int audit_log_buffer_write(audit_log_buffer_t *log, const char *buf,
+                           size_t len);
+void audit_log_buffer_pause(audit_log_buffer_t *log) noexcept;
+void audit_log_buffer_resume(audit_log_buffer_t *log) noexcept;
 
 #endif
