@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -20,24 +20,44 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#include "my_thread.h"          /* my_thread_id */
-#include "my_timer.h"           /* my_timer_t */
-#include "sql_class.h"          /* THD */
-#include "sql_timer.h"          /* thd_timer_set, etc. */
-#include "sql_parse.h"          /* Global_THD_manager, Find_thd_with_id */
-#include "mysqld.h"
+#include "sql/sql_timer.h"
 
-struct st_thd_timer_info
-{
+#include <stddef.h>
+#include <atomic>
+
+#include "my_dbug.h"
+#include "my_inttypes.h"
+#include "my_macros.h"
+#include "my_sys.h"
+#include "my_thread_local.h"
+#include "my_timer.h"  // my_timer_t
+#include "mysql/components/services/bits/mysql_mutex_bits.h"
+#include "mysql/psi/mysql_mutex.h"
+#include "mysql/service_mysql_alloc.h"
+#include "sql/mysqld.h"              // key_thd_timer_mutex
+#include "sql/mysqld_thd_manager.h"  // Global_THD_manager
+#include "sql/psi_memory_key.h"      // key_memory_thd_timer
+#include "sql/sql_class.h"           // THD
+#include "thr_mutex.h"
+
+/**
+  Cast a member of a structure to the structure that contains it.
+
+  @param  ptr     Pointer to the member.
+  @param  type    Type of the structure that contains the member.
+  @param  member  Name of the member within the structure.
+*/
+#define my_container_of(ptr, type, member) \
+  ((type *)((char *)ptr - offsetof(type, member)))
+
+struct THD_timer_info {
   my_thread_id thread_id;
   my_timer_t timer;
   mysql_mutex_t mutex;
   bool destroy;
 };
 
-C_MODE_START
 static void timer_callback(my_timer_t *);
-C_MODE_END
 
 /**
   Allocate and initialize a thread timer object.
@@ -45,35 +65,30 @@ C_MODE_END
   @return NULL on failure.
 */
 
-static THD_timer_info *
-thd_timer_create(void)
-{
+static THD_timer_info *thd_timer_create(void) {
   THD_timer_info *thd_timer;
-  DBUG_ENTER("thd_timer_create");
+  DBUG_TRACE;
 
-  thd_timer= (THD_timer_info *) my_malloc(key_memory_thd_timer,
-                                       sizeof(THD_timer_info),
-                                       MYF(MY_WME));
+  thd_timer = (THD_timer_info *)my_malloc(key_memory_thd_timer,
+                                          sizeof(THD_timer_info), MYF(MY_WME));
 
-  if (thd_timer == NULL)
-    DBUG_RETURN(NULL);
+  if (thd_timer == nullptr) return nullptr;
 
-  thd_timer->thread_id= 0;
+  thd_timer->thread_id = 0;
   mysql_mutex_init(key_thd_timer_mutex, &thd_timer->mutex, MY_MUTEX_INIT_FAST);
-  thd_timer->destroy= 0;
-  thd_timer->timer.notify_function= timer_callback;
+  thd_timer->destroy = false;
+  thd_timer->timer.notify_function = timer_callback;
 
   if (DBUG_EVALUATE_IF("thd_timer_create_failure", 0, 1) &&
-      ! my_timer_create(&thd_timer->timer))
-    DBUG_RETURN(thd_timer);
+      !my_timer_create(&thd_timer->timer))
+    return thd_timer;
 
   mysql_mutex_destroy(&thd_timer->mutex);
   my_free(thd_timer);
 
-  DBUG_RETURN(NULL);
+  return nullptr;
 }
 
-
 /**
   Notify a thread (session) that its timer has expired.
 
@@ -82,11 +97,10 @@ thd_timer_create(void)
   @return true if the object should be destroyed.
 */
 
-static bool
-timer_notify(THD_timer_info *thd_timer)
-{
+static bool timer_notify(THD_timer_info *thd_timer) {
   Find_thd_with_id find_thd_with_id(thd_timer->thread_id, false);
-  THD *thd= Global_THD_manager::get_instance()->find_thd(&find_thd_with_id);
+  THD_ptr thd_ptr =
+      Global_THD_manager::get_instance()->find_thd(&find_thd_with_id);
 
   assert(!thd_timer->destroy || !thd_timer->thread_id);
   /*
@@ -94,23 +108,19 @@ timer_notify(THD_timer_info *thd_timer)
     was being delivered. If this is the case, the timer object
     was detached (orphaned) and has no associated session (thd).
   */
-  if (thd)
-  {
+  if (thd_ptr) {
     /* process only if thread is not already undergoing any kill connection. */
-    if (thd->killed != THD::KILL_CONNECTION)
-    {
-      thd->awake(THD::KILL_TIMEOUT);
+    if (thd_ptr->killed != THD::KILL_CONNECTION) {
+      thd_ptr->awake(THD::KILL_TIMEOUT);
     }
-    mysql_mutex_unlock(&thd->LOCK_thd_data);
   }
 
   /* Mark the object as unreachable. */
-  thd_timer->thread_id= 0;
+  thd_timer->thread_id = 0;
 
   return thd_timer->destroy;
 }
 
-
 /**
   Timer expiration notification callback.
 
@@ -119,23 +129,19 @@ timer_notify(THD_timer_info *thd_timer)
   @note Invoked in a separate thread of control.
 */
 
-static void
-timer_callback(my_timer_t *timer)
-{
+static void timer_callback(my_timer_t *timer) {
   bool destroy;
   THD_timer_info *thd_timer;
 
-  thd_timer= my_container_of(timer, THD_timer_info, timer);
+  thd_timer = my_container_of(timer, THD_timer_info, timer);
 
   mysql_mutex_lock(&thd_timer->mutex);
-  destroy= timer_notify(thd_timer);
+  destroy = timer_notify(thd_timer);
   mysql_mutex_unlock(&thd_timer->mutex);
 
-  if (destroy)
-    thd_timer_destroy(thd_timer);
+  if (destroy) thd_timer_destroy(thd_timer);
 }
 
-
 /**
   Set the time until the currently running statement is aborted.
 
@@ -147,44 +153,42 @@ timer_callback(my_timer_t *timer)
   @return NULL on failure.
 */
 
-THD_timer_info *
-thd_timer_set(THD *thd, THD_timer_info *thd_timer, unsigned long time)
-{
-  DBUG_ENTER("thd_timer_set");
+THD_timer_info *thd_timer_set(THD *thd, THD_timer_info *thd_timer,
+                              unsigned long time) {
+  DBUG_TRACE;
 
   /* Create a new thread timer object if one was not provided. */
-  if (thd_timer == NULL && (thd_timer= thd_timer_create()) == NULL)
-    DBUG_RETURN(NULL);
+  if (thd_timer == nullptr && (thd_timer = thd_timer_create()) == nullptr)
+    return nullptr;
 
   assert(!thd_timer->destroy);
   assert(!thd_timer->thread_id);
 
   /* Mark the notification as pending. */
-  thd_timer->thread_id= thd->thread_id();
+  thd_timer->thread_id = thd->thread_id();
 
   /* Arm the timer. */
   if (DBUG_EVALUATE_IF("thd_timer_set_failure", 0, 1) &&
       !my_timer_set(&thd_timer->timer, time))
-    DBUG_RETURN(thd_timer);
+    return thd_timer;
 
   /* Dispose of the (cached) timer object. */
   thd_timer_destroy(thd_timer);
 
-  DBUG_RETURN(NULL);
+  return nullptr;
 }
 
-
 /**
   Reap a (possibly) pending timer object.
 
   @param  thd_timer   Thread timer object.
+  @param  pending     State of the timer object. true if timer is not expired.
+  false otherwise.
 
   @return true if the timer object is unreachable.
 */
 
-static bool
-reap_timer(THD_timer_info *thd_timer, bool pending)
-{
+static bool reap_timer(THD_timer_info *thd_timer, bool pending) {
   /* Cannot be tagged for destruction. */
   assert(!thd_timer->destroy);
 
@@ -196,9 +200,9 @@ reap_timer(THD_timer_info *thd_timer, bool pending)
     expiring. Otherwise, the timer notification function might be
     executing asynchronously in the context of a separate thread.
   */
-  bool unreachable= pending ? thd_timer->thread_id == 0 : true;
+  bool unreachable = pending ? thd_timer->thread_id == 0 : true;
 
-  thd_timer->thread_id= 0;
+  thd_timer->thread_id = 0;
 
   return unreachable;
 }
@@ -212,42 +216,36 @@ reap_timer(THD_timer_info *thd_timer, bool pending)
           Otherwise, the given timer object is returned.
 */
 
-THD_timer_info *
-thd_timer_reset(THD_timer_info *thd_timer)
-{
+THD_timer_info *thd_timer_reset(THD_timer_info *thd_timer) {
   bool unreachable;
-  int status, state;
-  DBUG_ENTER("thd_timer_cancel");
+  int status;
+  int state{0};
+  DBUG_TRACE;
 
-  status= my_timer_cancel(&thd_timer->timer, &state);
+  status = my_timer_cancel(&thd_timer->timer, &state);
 
   /*
     If the notification function cannot possibly run anymore, cache
     the timer object as there are no outstanding references to it.
   */
   mysql_mutex_lock(&thd_timer->mutex);
-  unreachable= reap_timer(thd_timer, status ? true : !state);
-  thd_timer->destroy= !unreachable;
+  unreachable = reap_timer(thd_timer, status ? true : !state);
+  thd_timer->destroy = !unreachable;
   mysql_mutex_unlock(&thd_timer->mutex);
 
-  DBUG_RETURN(unreachable ? thd_timer : NULL);
+  return unreachable ? thd_timer : nullptr;
 }
 
-
 /**
   Release resources allocated for a thread timer.
 
   @param  thd_timer   Thread timer object.
 */
 
-void
-thd_timer_destroy(THD_timer_info *thd_timer)
-{
-  DBUG_ENTER("thd_timer_destroy");
+void thd_timer_destroy(THD_timer_info *thd_timer) {
+  DBUG_TRACE;
 
   my_timer_delete(&thd_timer->timer);
   mysql_mutex_destroy(&thd_timer->mutex);
   my_free(thd_timer);
-
-  DBUG_VOID_RETURN;
 }
