@@ -19,28 +19,33 @@
 
 /* C++ standard header files */
 #include <array>
+#include <sstream>
 #include <string>
 #include <vector>
-#include <sstream>
 
 /* C standard header files */
 #include <ctype.h>
 
 /* MySQL header files */
 #include "my_dir.h"
+#include "sql/mysqld.h"
 
 /* MyRocks header files */
 #include "./ha_rocksdb.h"
 
+/* RocksDB includes */
+#include "rocksdb/env.h"
+#include "rocksdb/file_system.h"
+#include "rocksdb/io_status.h"
+
 namespace myrocks {
 
 /*
   Skip past any spaces in the input
 */
-const char *rdb_skip_spaces(const struct charset_info_st *const cs,
-                            const char *str) {
-  DBUG_ASSERT(cs != nullptr);
-  DBUG_ASSERT(str != nullptr);
+const char *rdb_skip_spaces(const CHARSET_INFO *const cs, const char *str) {
+  assert(cs != nullptr);
+  assert(str != nullptr);
 
   while (my_isspace(cs, *str)) {
     str++;
@@ -55,8 +60,8 @@ const char *rdb_skip_spaces(const struct charset_info_st *const cs,
   of characters in str2.
 */
 bool rdb_compare_strings_ic(const char *const str1, const char *const str2) {
-  DBUG_ASSERT(str1 != nullptr);
-  DBUG_ASSERT(str2 != nullptr);
+  assert(str1 != nullptr);
+  assert(str2 != nullptr);
 
   // Scan through the strings
   size_t ii;
@@ -79,9 +84,9 @@ const char *rdb_find_in_string(const char *str, const char *pattern,
   char quote = '\0';
   bool escape = false;
 
-  DBUG_ASSERT(str != nullptr);
-  DBUG_ASSERT(pattern != nullptr);
-  DBUG_ASSERT(succeeded != nullptr);
+  assert(str != nullptr);
+  assert(pattern != nullptr);
+  assert(succeeded != nullptr);
 
   *succeeded = false;
 
@@ -123,13 +128,13 @@ const char *rdb_find_in_string(const char *str, const char *pattern,
 /*
   See if the next valid token matches the specified string
 */
-const char *rdb_check_next_token(const struct charset_info_st *const cs,
-                                 const char *str, const char *const pattern,
+const char *rdb_check_next_token(const CHARSET_INFO *const cs, const char *str,
+                                 const char *const pattern,
                                  bool *const succeeded) {
-  DBUG_ASSERT(cs != nullptr);
-  DBUG_ASSERT(str != nullptr);
-  DBUG_ASSERT(pattern != nullptr);
-  DBUG_ASSERT(succeeded != nullptr);
+  assert(cs != nullptr);
+  assert(str != nullptr);
+  assert(pattern != nullptr);
+  assert(succeeded != nullptr);
 
   // Move past any spaces
   str = rdb_skip_spaces(cs, str);
@@ -147,10 +152,10 @@ const char *rdb_check_next_token(const struct charset_info_st *const cs,
 /*
   Parse id
 */
-const char *rdb_parse_id(const struct charset_info_st *const cs,
-                         const char *str, std::string *const id) {
-  DBUG_ASSERT(cs != nullptr);
-  DBUG_ASSERT(str != nullptr);
+const char *rdb_parse_id(const CHARSET_INFO *const cs, const char *str,
+                         std::string *const id) {
+  assert(cs != nullptr);
+  assert(str != nullptr);
 
   // Move past any spaces
   str = rdb_skip_spaces(cs, str);
@@ -209,10 +214,9 @@ const char *rdb_parse_id(const struct charset_info_st *const cs,
 /*
   Skip id
 */
-const char *rdb_skip_id(const struct charset_info_st *const cs,
-                        const char *str) {
-  DBUG_ASSERT(cs != nullptr);
-  DBUG_ASSERT(str != nullptr);
+const char *rdb_skip_id(const CHARSET_INFO *const cs, const char *str) {
+  assert(cs != nullptr);
+  assert(str != nullptr);
 
   return rdb_parse_id(cs, str, nullptr);
 }
@@ -220,14 +224,14 @@ const char *rdb_skip_id(const struct charset_info_st *const cs,
 /*
   Parses a given string into tokens (if any) separated by a specific delimiter.
 */
-const std::vector<std::string> parse_into_tokens(
-  const std::string& s, const char delim) {
+const std::vector<std::string> parse_into_tokens(const std::string &s,
+                                                 const char delim) {
   std::vector<std::string> tokens;
   std::string t;
   std::stringstream ss(s);
-
   while (getline(ss, t, delim)) {
-    tokens.push_back(t);
+    rdb_trim_whitespace_from_edges(t);
+    if (!t.empty()) tokens.push_back(t);
   }
 
   return tokens;
@@ -244,7 +248,7 @@ static const std::array<char, 16> rdb_hexdigit = {{'0', '1', '2', '3', '4', '5',
 */
 std::string rdb_hexdump(const char *data, const std::size_t data_len,
                         const std::size_t maxsize) {
-  DBUG_ASSERT(data != nullptr);
+  assert(data != nullptr);
 
   // Count the elements in the string
   std::size_t elems = data_len;
@@ -277,13 +281,22 @@ std::string rdb_hexdump(const char *data, const std::size_t data_len,
   return str;
 }
 
+// Return dir + '/' + file
+std::string rdb_concat_paths(const std::string &dir, const std::string &file) {
+  std::string result;
+  result.reserve(dir.length() + file.length() + 2);
+  result = dir;
+  result += FN_LIBCHAR;
+  result += file;
+  return result;
+}
+
 /*
   Attempt to access the database subdirectory to see if it exists
 */
 bool rdb_database_exists(const std::string &db_name) {
-  const std::string dir =
-      std::string(mysql_real_data_home) + FN_DIRSEP + db_name;
-  struct st_my_dir *const dir_info =
+  const auto dir = rdb_concat_paths(mysql_real_data_home, db_name);
+  MY_DIR *const dir_info =
       my_dir(dir.c_str(), MYF(MY_DONT_SORT | MY_WANT_STAT));
   if (dir_info == nullptr) {
     return false;
@@ -295,23 +308,83 @@ bool rdb_database_exists(const std::string &db_name) {
 
 void rdb_log_status_error(const rocksdb::Status &s, const char *msg) {
   if (msg == nullptr) {
-    // NO_LINT_DEBUG
-    sql_print_error("RocksDB: status error, code: %d, error message: %s",
+    LogPluginErrMsg(ERROR_LEVEL, 0, "Status error, code: %d, error message: %s",
                     s.code(), s.ToString().c_str());
     return;
   }
 
-  // NO_LINT_DEBUG
-  sql_print_error("RocksDB: %s, Status Code: %d, Status: %s", msg, s.code(),
-                  s.ToString().c_str());
+  LogPluginErrMsg(ERROR_LEVEL, 0, "%s, Status Code: %d, Status: %s", msg,
+                  s.code(), s.ToString().c_str());
 }
 
-void warn_about_bad_patterns(const Regex &regex, const char *name) {
+/*
+  Set the patterns string.  If there are invalid regex patterns they will
+  be stored in m_bad_patterns and the result will be false, otherwise the
+  result will be true.
+*/
+bool Regex_list_handler::set_patterns(
+    const std::string &pattern_str,
+    std::regex_constants::syntax_option_type flags) {
+  bool pattern_valid = true;
+
+  // Create a normalized version of the pattern string with all delimiters
+  // replaced by the '|' character
+  std::string norm_pattern = pattern_str;
+  std::replace(norm_pattern.begin(), norm_pattern.end(), m_delimiter, '|');
+
+  // Make sure no one else is accessing the list while we are changing it.
+  mysql_rwlock_wrlock(&m_rwlock);
+
+  // Clear out any old error information
+  m_bad_pattern_str.clear();
+
+  try {
+    // Replace all delimiters with the '|' operator and create the regex
+    // Note that this means the delimiter can not be part of a regular
+    // expression.  This is currently not a problem as we are using the comma
+    // character as a delimiter and commas are not valid in table names.
+
+    // std::regex implementation on MacOS X does not allow empty strings in
+    // constructors
+    if (norm_pattern.empty())
+      m_pattern.reset(new std::regex);
+    else
+      m_pattern.reset(new std::regex(norm_pattern, flags));
+  } catch (const std::regex_error &e) {
+    // This pattern is invalid.
+    pattern_valid = false;
+
+    // Put the bad pattern into a member variable so it can be retrieved later.
+    m_bad_pattern_str = pattern_str;
+  }
+
+  // Release the lock
+  mysql_rwlock_unlock(&m_rwlock);
+
+  return pattern_valid;
+}
+
+bool Regex_list_handler::matches(const std::string &str) const {
+  assert(m_pattern != nullptr);
+
+  // Make sure no one else changes the list while we are accessing it.
+  mysql_rwlock_rdlock(&m_rwlock);
+
+  // See if the table name matches the regex we have created
+  bool found = std::regex_match(str, *m_pattern);
+
+  // Release the lock
+  mysql_rwlock_unlock(&m_rwlock);
+
+  return found;
+}
+
+void warn_about_bad_patterns(const Regex_list_handler *regex_list_handler,
+                             const char *name) {
   // There was some invalid regular expression data in the patterns supplied
 
-  // NO_LINT_DEBUG
-  sql_print_warning("RocksDB: Invalid pattern in %s: %s", name,
-                    regex.pattern().c_str());
+  LogPluginErrMsg(WARNING_LEVEL, 0, "Invalid pattern in %s: %s", name,
+                  regex_list_handler->bad_pattern().c_str());
 }
 
 // Split a string based on a delimiter.  Two delimiters in a row will not add
@@ -325,43 +398,58 @@ std::vector<std::string> split_into_vector(const std::string &input,
   // Find next delimiter
   while ((pos = input.find(delimiter, start)) != std::string::npos) {
     // If there is any data since the last delimiter add it to the list
-    if (pos > start)
-      elems.push_back(input.substr(start, pos - start));
+    if (pos > start) elems.push_back(input.substr(start, pos - start));
 
     // Set our start position to the character after the delimiter
     start = pos + 1;
   }
 
   // Add a possible string since the last delimiter
-  if (input.length() > start)
-    elems.push_back(input.substr(start));
+  if (input.length() > start) elems.push_back(input.substr(start));
 
   // Return the resulting list back to the caller
   return elems;
 }
 
-bool rdb_check_rocksdb_corruption() {
-  return !my_access(myrocks::rdb_corruption_marker_file_name().c_str(), F_OK);
+bool rdb_has_rocksdb_corruption() {
+  rocksdb::DBOptions *rocksdb_db_options = get_rocksdb_db_options();
+  assert(rocksdb_db_options->env != nullptr);
+  const std::string fileName(myrocks::rdb_corruption_marker_file_name());
+
+  const auto &fs = rocksdb_db_options->env->GetFileSystem();
+  rocksdb::IOStatus io_s =
+      fs->FileExists(fileName, rocksdb::IOOptions(), nullptr);
+
+  return io_s.ok();
 }
 
 void rdb_persist_corruption_marker() {
-  const std::string& fileName = myrocks::rdb_corruption_marker_file_name();
-  int fd = my_open(fileName.c_str(), O_CREAT | O_SYNC, MYF(MY_WME));
-  if (fd < 0) {
-    sql_print_error("RocksDB: Can't create file %s to mark rocksdb as "
-                    "corrupted.",
+  rocksdb::DBOptions *rocksdb_db_options = get_rocksdb_db_options();
+  assert(rocksdb_db_options->env != nullptr);
+  const std::string fileName(myrocks::rdb_corruption_marker_file_name());
+
+  const auto &fs = rocksdb_db_options->env->GetFileSystem();
+  std::unique_ptr<rocksdb::FSWritableFile> file;
+  rocksdb::IOStatus io_s =
+      fs->NewWritableFile(fileName, rocksdb::FileOptions(), &file, nullptr);
+
+  if (!io_s.ok()) {
+    LogPluginErrMsg(ERROR_LEVEL, 0,
+                    "Can't create file %s to mark rocksdb as corrupted.",
                     fileName.c_str());
   } else {
-    sql_print_information("RocksDB: Creating the file %s to abort mysqld "
-                          "restarts. Remove this file from the data directory "
-                          "after fixing the corruption to recover. ",
-                          fileName.c_str());
+    LogPluginErrMsg(INFORMATION_LEVEL, 0,
+                    "Creating the file %s to abort mysqld restarts. Remove "
+                    "this file from the data directory after fixing the "
+                    "corruption to recover. ",
+                    fileName.c_str());
   }
 
-  int ret = my_close(fd, MYF(MY_WME));
-  if (ret) {
-    sql_print_error("RocksDB: Error (%d) closing the file %s", ret, fileName.c_str());
+  io_s = file->Close(rocksdb::IOOptions(), nullptr);
+  if (!io_s.ok()) {
+    LogPluginErrMsg(ERROR_LEVEL, 0, "Error (%s) closing the file %s",
+                    io_s.ToString().c_str(), fileName.c_str());
   }
 }
 
-} // namespace myrocks
+}  // namespace myrocks
